<?php

namespace Drupal\solrlog\Logger;

use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LogMessageParserInterface;
use Drupal\Core\Logger\RfcLoggerTrait;
use Drupal\solrlog\SolariumTrait;
use Psr\Log\LoggerInterface;

/**
 * Logs events in Search API.
 */
class SolrLogger implements LoggerInterface {
  use RfcLoggerTrait;
  use DependencySerializationTrait;

  use SolariumTrait;

  protected bool $serviceFunctinal = TRUE;

  /**
   * Number of runs within single page cycle.
   *
   * Used to distinguish document IDs.
   *
   * @var int
   */
  protected static int $runCount = 0;

  /**
   * SolrLogger constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Logger\LogMessageParserInterface $parser
   *   The parser to use when extracting message variables.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected LogMessageParserInterface $parser,
  ) {}

  /**
   * {@inheritdoc}
   */
  public function log($level, string|\Stringable $message, array $context = []): void {
    if (!$this->serviceFunctinal) {
      return;
    }
    // Remove backtrace and exception since they may contain
    // an unserializable variable.
    unset($context['exception']);
    $connector = $this->getConnector();
    if (!$connector) {
      return;
    }
    // Convert PSR3-style messages to \Drupal\Component\Render\FormattableMarkup
    // style, so they can be translated too in runtime.
    $message_placeholders = $this->parser->parseMessagePlaceholders($message, $context);
    $values = [
      'id' => \time() . (++self::$runCount),
      // This helps to clear/filter the documents.
      'ss_module_id' => 'solrlog',
      'itm_uid' => $context['uid'],
      'ss_type' => mb_substr($context['channel'], 0, 64),
      'tm_message' => $message,
      'tm_variables' => json_encode($message_placeholders, JSON_PRETTY_PRINT),
      'its_severity' => $level,
      'ss_link' => $context['link'],
      'ss_location' => $context['request_uri'],
      'ss_referer' => $context['referer'],
      'ss_hostname' => mb_substr($context['ip'], 0, 128),
      'dt_timestamp' => $context['timestamp'],
    ];
    $query = $connector->getUpdateQuery();
    $query->addDocument($query->createDocument($values))->addCommit();
    try {
      $connector->update($query);
    }
    catch (\Exception $e) {
      // Prevent infinite loop trying to log the message.
      $this->serviceFunctinal = FALSE;
    }
  }

}
